using System;
using System.Reflection;
using Advanced_Combat_Tracker;
using System.Windows.Forms;
using System.IO;
using System.Collections.Generic;
using System.Xml;
using System.Text.RegularExpressions;
using System.Text;
using System.Drawing;

[assembly: AssemblyTitle("Harvest Tracking")]
[assembly: AssemblyDescription("Parses and totals harvests")]
[assembly: AssemblyVersion("1.0.1.1")]

namespace Some_ACT_Plugin
{
    public class Harvest_Plugin : IActPluginV1
    {
        Label lblStatus;
        TabPage tpScreen;
        bool rareNext = false;
        Regex regHarvestAction = new Regex(@"\(\d{10}\)\[.{24}\] Vous (?<action>[^ ]+) (?<number>\d+) \\aITEM -?\d+ -?\d+:(?<itemName>[^\\]+)\\/a dans (?<nodeType>[^\.]+)\.", RegexOptions.Compiled);
        HarvestStats harvestStats = new HarvestStats();

        #region UI Components
        private GroupBox groupBox1;
        private ListBox listBox1;
        private TableLayoutPanel tableLayoutPanel1;
        private GroupBox groupBox2;
        private ListBox listBox2;
        private ListView listView1;
        private Label label1;
        private Panel panel1;
        private Button button1;
        private ContextMenuStrip menu1;
        private ToolStripMenuItem menuItemCsv;
        private ToolStripMenuItem menuItemHtml;
        private void InitUI()
        {
            this.groupBox1 = new GroupBox();
            this.listBox1 = new ListBox();
            this.tableLayoutPanel1 = new TableLayoutPanel();
            this.listBox2 = new ListBox();
            this.groupBox2 = new GroupBox();
            this.listView1 = new ListView();
            this.label1 = new Label();
            this.panel1 = new Panel();
            this.button1 = new Button();
            this.menu1 = new ContextMenuStrip();
            this.menuItemCsv = new ToolStripMenuItem();
            this.menuItemHtml = new ToolStripMenuItem();
            
            this.panel1.SuspendLayout();
            this.groupBox1.SuspendLayout();
            this.tableLayoutPanel1.SuspendLayout();
            this.groupBox2.SuspendLayout();
            tpScreen.SuspendLayout();

            this.groupBox1.Controls.Add(this.listBox1);
            this.groupBox1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.groupBox1.Padding = new System.Windows.Forms.Padding(6);
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "By Action/Node";

            this.listBox1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.listBox1.Sorted = true;
            this.listBox1.FormattingEnabled = true;
            this.listBox1.IntegralHeight = false;
            this.listBox1.TabIndex = 1;

            this.tableLayoutPanel1.ColumnCount = 2;
            this.tableLayoutPanel1.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, 200F));
            this.tableLayoutPanel1.ColumnStyles.Add(new ColumnStyle());
            this.tableLayoutPanel1.Controls.Add(this.button1, 0, 0);
            this.tableLayoutPanel1.Controls.Add(this.groupBox1, 0, 1);
            this.tableLayoutPanel1.Controls.Add(this.groupBox2, 0, 2);
            this.tableLayoutPanel1.Controls.Add(this.panel1, 1, 0);
            this.tableLayoutPanel1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.tableLayoutPanel1.RowCount = 3;
            this.tableLayoutPanel1.RowStyles.Add(new RowStyle(SizeType.Absolute, 32));
            this.tableLayoutPanel1.RowStyles.Add(new RowStyle(SizeType.Percent, 50F));
            this.tableLayoutPanel1.RowStyles.Add(new RowStyle(SizeType.Percent, 50F));
            this.tableLayoutPanel1.TabStop = false;

            this.listBox2.Dock = DockStyle.Fill;
            this.listBox2.Sorted = true;
            this.listBox2.FormattingEnabled = true;
            this.listBox2.IntegralHeight = false;
            this.listBox2.TabIndex = 2;

            this.groupBox2.Controls.Add(this.listBox2);
            this.groupBox2.Dock = DockStyle.Fill;
            this.groupBox2.Padding = new System.Windows.Forms.Padding(6);
            this.groupBox2.TabStop = false;
            this.groupBox2.Text = "By Item";

            this.listView1.Dock = DockStyle.Fill;
            this.listView1.FullRowSelect = true;
            this.listView1.GridLines = true;
            this.listView1.TabIndex = 3;
            this.listView1.UseCompatibleStateImageBehavior = false;
            this.listView1.View = View.Details;
            this.listView1.ContextMenuStrip = menu1;

            this.label1.Dock = DockStyle.Top;
            this.label1.Size = new Size(64, 18);
            this.label1.Text = "Data Displayed";
            this.label1.BackColor = Color.LightSteelBlue;
            this.label1.ForeColor = Color.MidnightBlue;
            this.label1.Font = new System.Drawing.Font("Tahoma", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label1.TextAlign = ContentAlignment.MiddleLeft;
            this.label1.BorderStyle = BorderStyle.Fixed3D;

            this.panel1.Dock = DockStyle.Fill;
            this.panel1.Padding = new Padding(0);
            this.tableLayoutPanel1.SetRowSpan(panel1, 3);
            this.panel1.Controls.Add(listView1);
            this.panel1.Controls.Add(label1);

            this.button1.Dock = DockStyle.Fill;
            this.button1.Text = "Return to Overview Display";
            this.button1.TabIndex = 0;

            this.menu1.Items.Add(menuItemCsv);
            this.menu1.Items.Add(menuItemHtml);

            this.menuItemCsv.Text = "Copy as CSV";

            this.menuItemHtml.Text = "Copy as HTML";

            this.groupBox1.ResumeLayout(false);
            this.tableLayoutPanel1.ResumeLayout(false);
            this.groupBox2.ResumeLayout(false);
            this.panel1.ResumeLayout(false);
            tpScreen.ResumeLayout(false);

            listBox1.Click += new EventHandler(listBox1_Click);
            listBox2.Click += new EventHandler(listBox2_Click);
            button1.Click += new EventHandler(button1_Click);
            menuItemCsv.Click += new EventHandler(menuItemCsv_Click);
            menuItemHtml.Click += new EventHandler(menuItemHtml_Click);
        }
        #endregion

        #region IActPluginV1 Members
        public void DeInitPlugin()
        {
            ActGlobals.oFormActMain.OnLogLineRead -= oFormActMain_OnLogLineRead;
            lblStatus.Text = "Plugin exited";
        }

        public void InitPlugin(TabPage pluginScreenSpace, Label pluginStatusText)
        {
            lblStatus = pluginStatusText;
            tpScreen = pluginScreenSpace;
            InitUI();
            tpScreen.Controls.Add(tableLayoutPanel1);
            lblStatus.Text = "Plugin started";
            ActGlobals.oFormActMain.OnLogLineRead += new LogLineEventDelegate(oFormActMain_OnLogLineRead);
        }
        #endregion
        void menuItemCsv_Click(object sender, EventArgs e)
        {
            if (listView1.Columns.Count == 0)
                return;
            StringBuilder table = new StringBuilder();
            StringBuilder row = new StringBuilder();
            for (int i = 0; i < listView1.Columns.Count; i++)
            {
                row.AppendFormat("{0},", listView1.Columns[i].Text);
            }
            table.AppendFormat("{0}\n", row.ToString(0, row.Length - 1));

            foreach (ListViewItem lvi in listView1.Items)
            {
                row = new StringBuilder();
                for (int i = 0; i < listView1.Columns.Count; i++)
                {
                    row.AppendFormat("{0},", lvi.SubItems[i].Text);
                }
                table.AppendFormat("{0}\n", row.ToString(0, row.Length - 1));
            }
            ActGlobals.oFormActMain.SendToClipboard(table.ToString(), true);
        }
        void menuItemHtml_Click(object sender, EventArgs e)
        {
            if (listView1.Columns.Count == 0)
                return;
            StringBuilder table = new StringBuilder();
            table.Append("<table>\n");
            StringBuilder row = new StringBuilder("<tr>");
            for (int i = 0; i < listView1.Columns.Count; i++)
            {
                row.AppendFormat("<th>{0}</th>", listView1.Columns[i].Text);
            }
            table.AppendFormat("{0}</tr>\n", row.ToString());

            foreach (ListViewItem lvi in listView1.Items)
            {
                row = new StringBuilder("<tr>");
                for (int i = 0; i < listView1.Columns.Count; i++)
                {
                    row.AppendFormat("<td>{0}</td>", lvi.SubItems[i].Text);
                }
                table.AppendFormat("{0}</tr>\n", row.ToString());
            }
            table.Append("</table>\n");
            ActGlobals.oFormActMain.SendHtmlToClipboard(table.ToString());
        }
        void button1_Click(object sender, EventArgs e)
        {
            listView1.BeginUpdate();
            PopulateLV0();
            listView1.EndUpdate();
        }
        void PopulateLV0()
        {
            listBox1.SelectedIndex = -1;
            listBox2.Items.Clear();
            listView1.Columns.Clear();
            listView1.Items.Clear();
            listView1.Columns.Add("Type");
            listView1.Columns.Add("# Collects");
            listView1.Columns.Add("% Collects");
            listView1.Columns.Add("# Items");
            listView1.Columns.Add("% Items");
            listView1.Columns.Add("# Rares");
            listView1.Columns.Add("% Rares");
            harvestStats.items.Sort();
            for (int i = 0; i < harvestStats.items.Count; i++)
            {
                NodeStats nsi = harvestStats.items[i];
                ListViewItem lvi = new ListViewItem(nsi.name);
                lvi.SubItems.Add(nsi.TotalCollects.ToString());
                lvi.SubItems.Add(String.Format("{0:0.00}%", nsi.PercentCollects));
                lvi.SubItems.Add(nsi.TotalItems.ToString());
                lvi.SubItems.Add(String.Format("{0:0.00}%", nsi.PercentItems));
                lvi.SubItems.Add(nsi.TotalRares.ToString());
                lvi.SubItems.Add(String.Format("{0:0.00}%", nsi.PercentRares));
                listView1.Items.Add(lvi);
            }
            ResizeLVCols();
            label1.Text = "Data Selected: Overview";
        }
        void listBox1_Click(object sender, EventArgs e)
        {
            listView1.BeginUpdate();
            PopulateLV1();
            listView1.EndUpdate();
        }
        private void PopulateLV1()
        {
            listBox2.Items.Clear();
            listView1.Columns.Clear();
            listView1.Items.Clear();
            if (listBox1.SelectedIndex > -1)
            {
                NodeStats nS = null;
                for (int i = 0; i < listBox1.Items.Count; i++)
                {
                    if (harvestStats.items[i].name == (string)listBox1.Items[listBox1.SelectedIndex])
                        nS = harvestStats.items[i];
                }
                for (int i = 0; i < nS.items.Count; i++)
                {
                    listBox2.Items.Add(nS.items[i].name);
                }

                listView1.Columns.Clear();
                listView1.Items.Clear();
                listView1.Columns.Add("Type");
                listView1.Columns.Add("# Collects");
                listView1.Columns.Add("% Collects");
                listView1.Columns.Add("# Items");
                listView1.Columns.Add("% Items");
                listView1.Columns.Add("Avg Items/Collect");
                listView1.Columns.Add("# Rares");
                listView1.Columns.Add("% Rares");
                nS.items.Sort();
                for (int i = 0; i < nS.items.Count; i++)
                {
                    ItemStats iS = nS.items[i];
                    ListViewItem lvi = new ListViewItem(iS.name);
                    lvi.SubItems.Add(iS.TotalCollects.ToString());
                    lvi.SubItems.Add(String.Format("{0:0.00}%", iS.PercentCollects));
                    lvi.SubItems.Add(iS.TotalItems.ToString());
                    lvi.SubItems.Add(String.Format("{0:0.00}%", iS.PercentItems));
                    lvi.SubItems.Add(String.Format("{0:0.00}", iS.AvgItemsPerCollect));
                    lvi.SubItems.Add(iS.TotalRares.ToString());
                    lvi.SubItems.Add(String.Format("{0:0.00}%", iS.PercentRares));
                    listView1.Items.Add(lvi);
                }
                ResizeLVCols();
                label1.Text = String.Format("Data Selected: {0}", nS.name);
            }
        }
        void listBox2_Click(object sender, EventArgs e)
        {
            listView1.BeginUpdate();
            PopulateLV2();
            listView1.EndUpdate();
        }
        private void PopulateLV2()
        {
            listView1.Columns.Clear();
            listView1.Items.Clear();
            if (listBox2.SelectedIndex > -1)
            {
                NodeStats nS = null;
                for (int i = 0; i < listBox1.Items.Count; i++)
                {
                    if (harvestStats.items[i].name == (string)listBox1.Items[listBox1.SelectedIndex])
                        nS = harvestStats.items[i];
                }
                ItemStats iS = null;
                for (int i = 0; i < listBox2.Items.Count; i++)
                {
                    if (nS.items[i].name == (string)listBox2.Items[listBox2.SelectedIndex])
                        iS = nS.items[i];
                }

                listView1.Columns.Clear();
                listView1.Items.Clear();
                listView1.Columns.Add("Time");
                listView1.Columns.Add("Action");
                listView1.Columns.Add("Amount");
                listView1.Columns.Add("Item Name");
                listView1.Columns.Add("Node Name");
                listView1.Columns.Add("Rare");
                iS.items.Sort();
                for (int i = 0; i < iS.items.Count; i++)
                {
                    HarvestData hD = iS.items[i];
                    ListViewItem lvi = new ListViewItem(hD.time.ToLongTimeString());
                    lvi.SubItems.Add(hD.actionVerb);
                    lvi.SubItems.Add(hD.amount.ToString());
                    lvi.SubItems.Add(hD.itemName);
                    lvi.SubItems.Add(hD.nodeType);
                    lvi.SubItems.Add(hD.rare.ToString());
                    listView1.Items.Add(lvi);
                }
                ResizeLVCols();
                label1.Text = String.Format("Data Selected: {0} - {1}", iS.parent.name, iS.name);
            }
        }
        private void ResizeLVCols()
        {
            try
            {
                int totalChrs = 0;
                int[] chrNums = new int[listView1.Columns.Count];
                int[] chrMax = new int[listView1.Columns.Count];

                if (listView1.Items.Count == 0)
                {
                    for (int i = 0; i < listView1.Columns.Count; i++)
                    {
                        chrNums[i] += 10;
                        totalChrs += 10;
                    }
                }
                else
                {
                    for (int j = 0; j < listView1.Items.Count; j++)
                    {
                        ListViewItem lvi = listView1.Items[j];
                        for (int i = 0; i < listView1.Columns.Count; i++)
                        {
                            if (chrMax[i] < lvi.SubItems[i].Text.Length)
                                chrMax[i] = lvi.SubItems[i].Text.Length;
                            int total = lvi.SubItems[i].Text.Length + (listView1.Columns[i].Text.Length);
                            chrNums[i] += total;
                            totalChrs += total;
                        }
                    }
                    for (int i = 0; i < listView1.Columns.Count; i++)
                    {
                        chrNums[i] += chrMax[i] * listView1.Items.Count;
                        totalChrs += chrMax[i] * listView1.Items.Count;
                    }
                }
                for (int i = 0; i < listView1.Columns.Count; i++)
                {
                    ColumnHeader cH = listView1.Columns[i];
                    cH.Width = Convert.ToInt32((listView1.Width - 24) * ((float)chrNums[i] / totalChrs));
                }
            }
            catch { }
        }
        void oFormActMain_OnLogLineRead(bool isImport, LogLineEventArgs logInfo)
        {
            if (logInfo.detectedType == 0)
            {
                if (logInfo.logLine.EndsWith("Vous avez trouvé un objet rare !"))
                    rareNext = true;
                if (regHarvestAction.IsMatch(logInfo.logLine))
                {
                    harvestStats.AddHarvest(new HarvestData(logInfo.detectedTime, 
                        regHarvestAction.Replace(logInfo.logLine,"$1"), Int32.Parse(regHarvestAction.Replace(logInfo.logLine, "$2")),
                        regHarvestAction.Replace(logInfo.logLine, "$3"), regHarvestAction.Replace(logInfo.logLine, "$4"), rareNext));
                    rareNext = false;
                    PopulateLBs();
                }
            }
        }
        void PopulateLBs()
        {
            for (int i = 0; i < harvestStats.items.Count; i++)
            {
                if (!listBox1.Items.Contains(harvestStats.items[i].name))
                    listBox1.Items.Add(harvestStats.items[i].name);
            }
            if (listBox1.SelectedIndex > -1)
            {
                NodeStats ns = null;
                string selected = (string)listBox1.Items[listBox1.SelectedIndex];
                for (int i = 0; i < harvestStats.items.Count; i++)
                {
                    if (harvestStats.items[i].name == selected)
                    {
                        ns = harvestStats.items[i];
                        break;
                    }
                }
                for (int i = 0; i < harvestStats.items.Count; i++)
                {
                    if (!listBox2.Items.Contains(ns.items[i].name))
                        listBox2.Items.Add(ns.items[i].name);
                }
            }
        }
        private class HarvestData : IComparable<HarvestData>
        {
            public DateTime time;
            public string actionVerb;
            public int amount;
            public string itemName;
            public string nodeType;
            public bool rare;
            public HarvestData(DateTime Time, string ActionVerb, int Amount, string ItemName, string NodeType, bool Rare)
            {
                this.time = Time;
                this.actionVerb = ActionVerb;
                this.amount = Amount;
                this.itemName = ItemName;
                this.nodeType = NodeType;
                this.rare = Rare;
            }

            #region IComparable<HarvestData> Members

            public int CompareTo(HarvestData other)
            {
                return this.time.CompareTo(other.time);
            }

            #endregion
        }
        private class ItemStats : IEquatable<ItemStats>, IComparable<ItemStats>
        {
            public string name;
            public NodeStats parent;
            public List<HarvestData> items = new List<HarvestData>();
            public ItemStats(string Name, NodeStats Parent)
            {
                name = Name;
                parent = Parent;
            }
            public void AddHarvest(HarvestData item)
            {
                items.Add(item);
            }
            public int TotalCollects
            {
                get
                {
                    return items.Count;
                }
            }
            public int TotalItems
            {
                get
                {
                    int total = 0;
                    for (int i = 0; i < items.Count; i++)
                    {
                        total += items[i].amount;
                    }
                    return total;
                }
            }
            public int TotalRares
            {
                get
                {
                    int total = 0;
                    for (int i = 0; i < items.Count; i++)
                    {
                        if (items[i].rare)
                            total++;
                    }
                    return total;
                }
            }
            public float AvgItemsPerCollect
            {
                get
                {
                    return (float)TotalItems / (float)TotalCollects;
                }
            }
            public float PercentCollects
            {
                get
                {
                    return ((float)TotalCollects / (float)parent.TotalCollects) * 100F;
                }
            }
            public float PercentItems
            {
                get
                {
                    return ((float)TotalItems / (float)parent.TotalItems) * 100F;
                }
            }
            public float PercentRares
            {
                get
                {
                    return ((float)TotalRares / (float)parent.TotalRares) * 100F;
                }
            }

            #region IEquatable<ItemStats> Members
            public bool Equals(ItemStats other)
            {
                return this.name.Equals(other.name);
            }
            #endregion
            #region IComparable<ItemStats> Members
            public int CompareTo(ItemStats other)
            {
                return this.name.CompareTo(other.name);
            }
            #endregion
        }
        private class NodeStats : IEquatable<NodeStats>, IComparable<NodeStats>
        {
            public string name;
            public HarvestStats parent;
            public List<ItemStats> items = new List<ItemStats>();
            public NodeStats(string Name, HarvestStats Parent)
            {
                name = Name;
                parent = Parent;
            }
            public void AddHarvest(HarvestData item)
            {
                ItemStats all = new ItemStats(" All", this);
                int index = items.IndexOf(all);
                if (index > -1)
                    all = items[index];
                else
                    items.Add(all);
                ItemStats unique = new ItemStats(item.itemName, this);
                index = items.IndexOf(unique);
                if (index > -1)
                    unique = items[index];
                else
                    items.Add(unique);

                all.AddHarvest(item);
                unique.AddHarvest(item);
            }
            public int TotalCollects
            {
                get
                {
                    for (int i = 0; i < items.Count; i++)
                    {
                        if (items[i].name == " All")
                        {
                            return items[i].TotalCollects;
                        }
                    }
                    return 0;
                }
            }
            public int TotalItems
            {
                get
                {
                    for (int i = 0; i < items.Count; i++)
                    {
                        if (items[i].name == " All")
                        {
                            return items[i].TotalItems;
                        }
                    }
                    return 0;
                }
            }
            public int TotalRares
            {
                get
                {
                    for (int i = 0; i < items.Count; i++)
                    {
                        if (items[i].name == " All")
                        {
                            return items[i].TotalRares;
                        }
                    }
                    return 0;
                }
            }
            public float PercentCollects
            {
                get
                {
                    return ((float)TotalCollects / (float)parent.TotalCollects) * 100F;
                }
            }
            public float PercentItems
            {
                get
                {
                    return ((float)TotalItems / (float)parent.TotalItems) * 100F;
                }
            }
            public float PercentRares
            {
                get
                {
                    return ((float)TotalRares / (float)parent.TotalRares) * 100F;
                }
            }

            #region IEquatable<NodeStats> Members
            public bool Equals(NodeStats other)
            {
                return this.name.Equals(other.name);
            }
            #endregion
            #region IComparable<NodeStats> Members
            public int CompareTo(NodeStats other)
            {
                return this.name.CompareTo(other.name);
            }
            #endregion
        }
        private class HarvestStats
        {
            public List<NodeStats> items = new List<NodeStats>();

            public void AddHarvest(HarvestData item)
            {
                NodeStats all = new NodeStats(" All", this);
                int index = items.IndexOf(all);
                if (index > -1)
                    all = items[index];
                else
                    items.Add(all);
                NodeStats uniqueAction = new NodeStats(item.actionVerb, this);
                index = items.IndexOf(uniqueAction);
                if (index > -1)
                    uniqueAction = items[index];
                else
                    items.Add(uniqueAction);
                NodeStats uniqueNode = new NodeStats(item.nodeType, this);
                index = items.IndexOf(uniqueNode);
                if (index > -1)
                    uniqueNode = items[index];
                else
                    items.Add(uniqueNode);

                all.AddHarvest(item);
                uniqueAction.AddHarvest(item);
                uniqueNode.AddHarvest(item);
            }
            public int TotalCollects
            {
                get
                {
                    for (int i = 0; i < items.Count; i++)
                    {
                        if (items[i].name == " All")
                        {
                            return items[i].TotalCollects;
                        }
                    }
                    return 0;
                }
            }
            public int TotalItems
            {
                get
                {
                    for (int i = 0; i < items.Count; i++)
                    {
                        if (items[i].name == " All")
                        {
                            return items[i].TotalItems;
                        }
                    }
                    return 0;
                }
            }
            public int TotalRares
            {
                get
                {
                    for (int i = 0; i < items.Count; i++)
                    {
                        if (items[i].name == " All")
                        {
                            return items[i].TotalRares;
                        }
                    }
                     return 0;
               }
            }
        }
    }
}
